#!/usr/local/bin/python
# -*- coding: UTF-8 -*-
#---------------------------
# Name: mythvidexport.py
# Python Script
# Author: Raymond Wagner
# Purpose
#   This python script is intended to function as a user job, run through
#   mythjobqueue, capable of exporting recordings into MythVideo.
#---------------------------
__title__  = "MythVidExport"
__author__ = "Raymond Wagner"
__version__= "v0.5.0"

usage_txt = """
This script can be run from the command line, or called through the mythtv
jobqueue.  The input format will be:
  mythvidexport.py [options] <--chanid <channel id>> <--starttime <start time>>
                 --- or ---
  mythvidexport.py [options] %JOBID%

Options are:
        --mformat <format string>
        --tformat <format string>
        --gformat <format string>
            overrides the stored format string for a single run
        --listingonly
            use EPG data rather than grabbers for metadata
            will still try to grab episode and season information from ttvdb.py

Additional functions are available beyond exporting video
  mythvidexport.py <options>
        -h, --help             show this help message
        -p, --printformat      print existing format strings
        -f, --helpformat       lengthy description for formatting strings
        --mformat <string>     replace existing Movie format
        --tformat <string>     replace existing TV format
        --gformat <string>     replace existing Generic format
"""

from MythTV import MythTV, MythDB, MythLog, MythVideo, Program, FileTransfer, Job
#from MythTV.MythLog import EMERGENCY, ALERT, CRITICAL, ERROR, WARNING, NOTICE, INFO, DEBUG, ALL
from socket import gethostname
from urllib import urlretrieve
from optparse import OptionParser
import sys, re, os, time


#log = MythLog(NOTICE, 'MythVidExport.py')

class VIDEO:
	job = None
	dest = None

	viddata = {}
	cast = []
	genre = []
	country = []

	def __init__(self, opts, jobid):
		if jobid:
			self.job = Job(jobid)
			self.chanid = self.job.chanid
			self.rtime = int("%04d%02d%02d%02d%02d%02d" % self.job.starttime.timetuple()[0:6])
			self.jobhost = self.job.host
			self.job.setStatus(3)
		else:
			self.chanid = opts.chanid
			self.rtime = opts.starttime
			self.jobhost = gethostname()

		self.opts = opts
		self.mythdb = MythDB()
		self.cursor = self.mythdb.cursor()

		# load grabber scripts
		self.get_grabber()

		# load formatting strings
		self.get_fmt()
		if opts.tformat:
			selt.tfmt = opts.tformat
		if opts.mformat:
			self.mfmt = opts.mformat
		if opts.gformat:
			self.gfmt = opts.gformat

		# process file
		self.get_source()
		self.get_meta()
		self.get_dest()

		# save file
		self.copy()
		self.write_meta()
		self.seekdata()

	def get_source(self):
		## connects to myth data socket and finds recording
		mythinst = MythTV()
		self.prog = mythinst.getRecording(self.chanid,self.rtime)

		self.srchost = self.prog.hostname
		self.source = mythinst.getCheckfile(self.prog)

	def get_grabber(self):
		# get tv grabber command
		self.tvgrabber = self.mythdb.getSetting('mythvideo.TVGrabber', self.jobhost)

		# insert optional ttvdb config file if using ttvdb grabber
		if os.access(os.path.expanduser("~/.mythtv/ttvdb.conf"),os.F_OK):
			ttvdbconf = os.path.expanduser("~/.mythtv/ttvdb.conf")
			if self.tvgrabber.split('/')[-1] == "ttvdb.py":
				self.tvgrabber += ' -c '+ttvdbconf

		# grab tmdb commands
		self.moviegrabber = self.mythdb.getSetting('mythvideo.MovieGrabber', self.jobhost)

	def get_meta(self):
		if self.prog.subtitle:  # subtitle exists, assume tv show
			self.get_ttvdb()
		else:                   # assume movie
			self.get_tmdb()

	def get_ttvdb(self):
		## grab season and episode number from thetvdb.com, exit if failed
		fp = os.popen("""%s -N "%s" "%s" """ % (self.tvgrabber,self.prog.title,self.prog.subtitle),'r')
		match = re.search("S(?P<season>[0-9]*)E(?P<episode>[0-9]*)", fp.read())
		fp.close()
		if match is None:
#			log.warning('Grabber error', "TTVDB.py failed to get season/episode numbers for %s - %s. Reverting to generic export." % (self.prog.title, self.prog.subtitle))
			self.get_generic()
			return

		self.viddata['season'] = int(match.group('season'))
		self.viddata['episode'] = int(match.group('episode'))

		if self.opts.listingonly:  # do not grab subsequent information from ttvdb
			self.get_generic()

		else: # gather remaining data
			self.process_meta("""%s -D "%s" %d %d""" % (self.tvgrabber,self.prog.title,self.viddata['season'],self.viddata['episode']))
			self.viddata['title'] = self.prog.title
			if 'fanart' in self.viddata:
				fanart = self.viddata['fanart'].split(',')
				self.viddata['fanart'] = fanart[self.viddata['season']%len(fanart)]
			for type in ('coverfile','screenshot','banner'):
                                if type in self.viddata:
                                        self.viddata[type] = self.viddata[type].split(',')[0]

		# set some common data		
		self.country = ()
		self.viddata['showlevel'] = 1
		self.type = 'TV'

	def get_tmdb(self):
		if self.opts.listingonly:  # do not grab from tmdb
			self.get_generic()
		else: # find inetref from tmdb
			match = None
			fp = os.popen("""%s "%s" """ % (self.tmdb['List'], self.prog.title))
			if self.prog.year:
				match = re.search("(?P<inetref>[0-9]*):%s[ ]*\(%s\)" % (self.prog.title, self.prog.year), fp.read())
			else:
				match = re.search("(?P<inetref>[0-9]*):%s[ ]*\([0-9]*\)" % self.prog.title, fp.read())
			fp.close()
			if match is None:
#				log.warning('Grabber error', "TMDB.py failed to find matching movie for %s. Reverting to generic export." % self.prog.title)
				self.get_generic()
				return

			# gather remanining data
			self.viddata['inetref'] = match.group('inetref')
			self.process_meta("%s -D %s" % (self.moviegrabber, match.group('inetref')))
			for type in ('coverfile','screenshot','banner','fanart'):
				if type in self.viddata:
					self.viddata[type] = self.viddata[type].split(',')[0]


		# set some common data
		self.country = ()
		self.viddata['showlevel'] = 1
		self.type = 'MOVIE'

	def get_generic(self):
		# pull available program data
		self.viddata['title'] = self.prog.title
		if self.prog.subtitle:
			self.viddata['subtitle'] = self.prog.subtitle
		if self.prog.description:
			self.viddata['plot'] = self.prog.description
		if self.prog.year:
			self.viddata['year'] = self.prog.year
		self.viddata['length'] = str(int((self.prog.recendts-self.prog.recstartts).seconds/60))
		self.viddata['inetref'] = '00000000'
		self.viddata['director'] = self.mythdb.getCast(self.chanid, self.rtime, roles='director')
		if len(self.viddata['director']) == 0:
			self.viddata['director'] = 'NULL'
		else:
			self.viddata['director'] = self.viddata['director'][0]
		self.cast = self.mythdb.getCast(self.chanid, self.rtime, roles=('actor','guest_star','host','commentator','guest'))
		self.country = ()
		self.viddata['showlevel'] = 1
		self.viddata['rating'] = 'NR'
		self.viddata['coverfile'] = 'No Cover'
		self.type = 'GENERIC'
		

	def process_meta(self, command):
		# process grabber command and pull available data
		fp = os.popen(command)
		res = fp.read()
		fp.close()
		dict = {}
		for point in res.split('\n')[:-1]:
			if point.find(':') == -1:
				continue
			key,dat = point.split(':',1)
			dict[key] = dat
		for (dbkey, datkey) in (('title','Title'),('subtitle','Subtitle'),('director','Director'),
					('plot','Plot'),('inetref','Seriesid'), ('year','Year'),
					('userrating','UserRating'),('length','Runtime'),('inetref','InetRef'),
					('screenshot','Screenshot'),('fanart','Fanart'),('banner','Banner'),
					('coverfile','Coverart'),('homepage','URL')):
			if datkey in dict.keys():
				self.viddata[dbkey] = dict[datkey].strip()
		if 'Cast' in dict.keys():
			self.cast = tuple(dict['Cast'].split(', '))
		if 'Genres' in dict.keys():
			self.genre = tuple(dict['Genres'].split(', '))
		if 'coverfile' not in self.viddata:
			self.viddata['coverfile'] = 'No Cover'
		if 'rating' not in self.viddata:
			self.viddata['rating'] = 'NR'


	def get_dest(self):
		
		subpath = None
		if self.type == 'TV':
			subpath = self.process_fmt(self.tfmt)
		elif self.type == 'MOVIE':
			subpath = self.process_fmt(self.mfmt)
		elif self.type == 'GENERIC':
			subpath = self.process_fmt(self.gfmt)
		subpath += self.source[self.source.rfind('.'):]

		res = self.mythdb.getStorageGroup('Videos',self.jobhost)
		if len(res):
			self.dest = res[0]['dirname']+subpath
			self.destdb = subpath
		else:
			self.dest = self.mythdb.getSetting("VideoStartupDir",self.jobhost)+'/'+subpath
			self.destdb = self.dest

		tmppath = self.dest[0:self.dest.rfind('/')] 
		if not os.access(tmppath,os.F_OK):
			os.makedirs(tmppath)

	def get_fmt(self):
		self.tfmt = self.mythdb.getSetting('mythvideo.TVexportfmt')
		if not self.tfmt:
			self.tfmt = 'Television/%TITLE%/Season %SEASON%/%TITLE% - S%SEASON%E%EPISODEPAD% - %SUBTITLE%'
		self.mfmt = self.mythdb.getSetting('mythvideo.MOVIEexportfmt')
		if not self.mfmt:
			self.mfmt = 'Movies/%TITLE%'
		self.gfmt = self.mythdb.getSetting('mythvideo.GENERICexportfmt')
		if not self.gfmt:
			self.gfmt = 'Videos/%TITLE%'

	def process_fmt(self, fmt):
		# replace fields from viddata
		rep = ( ('%TITLE%','title','%s'),('%SUBTITLE%','subtitle','%s'),
			('%SEASON%','season','%d'),('%SEASONPAD%','season','%02d'),
			('%EPISODE%','episode','%d'),('%EPISODEPAD%','episode','%02d'),
			('%YEAR%','year','%s'),('%DIRECTOR%','director','%s'))
		for (tag, data, format) in rep:
			if data in self.viddata.keys():
				fmt = fmt.replace(tag,format % self.viddata[data])
			else:
				fmt = fmt.replace(tag,'')

		# replace fields from program data
		rep = ( ('%HOSTNAME','hostname','%s'),('%STORAGEGROUP%','storagegroup','%s'))
		for (tag, data, format) in rep:
			data = eval('self.prog.%s' % data)
			fmt = fmt.replace(tag,format % data)

#		fmt = fmt.replace('%CARDID%',self.prog.cardid)
#		fmt = fmt.replace('%CARDNAME%',self.prog.cardid)
#		fmt = fmt.replace('%SOURCEID%',self.prog.cardid)
#		fmt = fmt.replace('%SOURCENAME%',self.prog.cardid)
#		fmt = fmt.replace('%CHANNUM%',self.prog.channum)
#		fmt = fmt.replace('%CHANNAME%',self.prog.cardid)

		if len(self.genre):
			fmt = fmt.replace('%GENRE%',self.genre[0])
		else:
			fmt = fmt.replace('%GENRE%','')
#		if len(self.country):
#			fmt = fmt.replace('%COUNTRY%',self.country[0])
#		else:
#			fmt = fmt.replace('%COUNTRY%','')
		return fmt

	def copy(self):
		if self.opts.skip:
			return
		if self.opts.sim:
			print 'Transferring from "%s:%s" to "%s"' % (self.srchost,self.source,self.dest)
			return
		srcp = None
		dtime = None

#		log.notice('Starting transfer','from "%s:%s" to "%s"' % (self.srchost,self.source,self.dest))
		stime = time.time()
		ctime = time.time()
		if os.access(self.source,os.F_OK):
			srcp = open(self.source,'r')
		else:
			srcp = FileTransfer(self.prop)
		destp = open(self.dest,'w')
		srcsize = self.prog.filesize
		destsize = [0,0,0,0,0,0,0,0,0,0]
		if self.job:
			self.job.setStatus(4)
		tsize = 2**18
		while tsize == 2**18:
			if (srcsize - destp.tell()) < tsize:
				tsize = srcsize - destp.tell()
			if time.time() - ctime > 4:
				ctime = time.time()
				destsize.append(destp.tell())
				rem = srcsize - destp.tell()
				if destsize[0]:
					dtime = 40
				else:
					dtime = int(ctime - stime)
				rate = (destp.tell() - destsize.pop(0))/dtime
				remt = rem/rate
				if self.job:
					self.job.setComment("%02d%% complete - %s seconds remaining" % (destsize[9]*100/srcsize, remt))
			destp.write(srcp.read(tsize))

		srcp.close()
		destp.close()
#		log.notice('Transfer complete','%d seconds elapsed' % int(time.time()-stime))
		if self.job:
			self.job.setComment("Complete - %d seconds elapsed" % (int(time.time()-stime)))
			self.job.setStatus(256)


	def write_meta(self):
		mythvid = MythVideo()
		self.viddata['filename'] = self.destdb
		self.viddata['host'] = self.jobhost
		self.write_images()

		intid = mythvid.getMetadataId(self.destdb)
		if intid:
			if self.opts.sim:
				print "Existing entry found at [%d], updating with:" % intid
				print self.viddata
			else:
				mythvid.setMetadata(self.viddata,intid)
		else:
			if self.opts.sim:
				print "Creating new entry using:"
				print self.viddata
			else:
				intid = mythvid.setMetadata(self.viddata)
		for name in self.cast:
			if self.opts.sim:
				print "Adding cast member: " +name
			else:
				mythvid.setCast(name, intid)
		if self.job:
			self.job.setStatus(272)

	def write_images(self):
		path = None
		dbpath = None
		hash = {'coverfile':('Coverart','VideoArtworkDir','coverart'),
			'banner':('Banners','mythvideo.bannerDir','banner'),
			'fanart':('Fanart','mythvideo.fanartDir','fanart'),
			'screenshot':('Screenshots','mythvideo.screenshotDir','screenshot')}
		for mode in hash.keys():
			if mode not in self.viddata:
				if self.opts.sim:
					print "No %s found" % mode
				continue
			url = self.viddata[mode]
			if url == 'No Cover' or url == '':
				if self.opts.sim:
					print "No %s found" % mode
				continue
			if self.type == 'TV':
				if mode == 'screenshot':
					file = '%s Season %dx%d_%s.%s' % (self.viddata['title'],
									self.viddata['season'],
									self.viddata['episode'],
									hash[mode][2],
									url[url.rfind('.')+1:])
				else:
					file = '%s Season %d_%s.%s' % (self.viddata['title'],
									self.viddata['season'],
									hash[mode][2],
									url[url.rfind('.')+1:])
			else:
				file = '%s_%s.%s' % (self.viddata['title'],
							hash[mode][2],
							url[url.rfind('.')+1:])

			res = self.mythdb.getStorageGroup(hash[mode][0],self.jobhost)
			if len(res):
				path = res[0]['dirname']+file
				self.viddata[mode] = file
			else:
				path = self.mythdb.getSetting(hash[mode][1],self.jobhost)+'/'+file
				self.viddata[mode] = path

			if self.opts.sim:
				print "Found %s at %s" % (mode,self.image[mode])
				return

			if not os.access(path,os.F_OK):
				urlretrieve(url,path)
			if (mode == 'coverfile') & (self.type == 'TV'):
				vdir = self.dest[:self.dest.rfind('/')]
				if not os.access(vdir+'folder.jpg',os.F_OK):
					urlretrieve(url,vdir+'/folder.jpg')

	def seekdata(self):
		if self.opts.sim:
			print "Running \"mythcommflag --rebuild --video '%s'\"" % self.dest
			return
		fp = os.popen("mythcommflag --rebuild --video '%s'" % self.dest)
		fp.close()

def usage():
	print("mythvidexport.py [options] [--chanid=<chanid> --starttime=<starttime> | <jobid>]")
	print("        This script can be run by specifing the channel and start time directly")
	print("            or by specifing the ID of a job in jobqueue")
	print("")
	print("        Run from the command line through the former:")
	print("            mythvidexport.py --chanid=1002 --starttime=200907010000")
        print("        Or from a user script through the latter:")
        print("            mythvidexport.py %JOBID%")
	print("")
	print("    Options:")
	print("        -h/--help and -f/--helpformat:")
	print("             return this help, or a listing of available formatting strings")
	print("        --fformat='<string>' and --dformat='<string>':")
	print("             override the stored formatting string in the database")
        print("             if no recording is specified, store format string to the database")

def usage_format():
	print("The default strings are:")
	print("    Television: 'Television/%TITLE%/Season %SEASON%/%TITLE% - S%SEASON%E%EPISODEPAD% - %SUBTITLE%'")
	print("    Movie:      'Movies/%TITLE%'")
	print("    Generic:    'Videos/%TITLE%'")
	print("")
	print("Available strings:")
	print("    %TITLE%:         series title")
	print("    %SUBTITLE%:      episode title")
	print("    %SEASON%:        season number")
	print("    %SEASONPAD%:     season number, padded to 2 digits")
	print("    %EPISODE%:       episode number")
	print("    %EPISODEPAD%:    episode number, padded to 2 digits")
	print("    %YEAR%:          year")
	print("    %DIRECTOR%:      director")
#	print("    %CARDID%:        ID of tuner card used to record show")
#	print("    %CARDNAME%:      name of tuner card used to record show")
#	print("    %SOURCEID%:      ID of video source used to record show")
#	print("    %SOURCENAME%:    name of video source used to record show")
        print("    %HOSTNAME%:      backend used to record show")
	print("    %STORAGEGROUP%:  storage group containing recorded show")
#	print("    %CHANNUM%:       ID of channel used to record show")
#	print("    %CHANNAME%:      name of channel used to record show")
	print("    %GENRE%:         first genre listed for recording")
#	print("    %COUNTRY%:       first country listed for recording")

def print_format():
	db = MythDB()
	tfmt = db.getSetting('mythvideo.TVexportfmt')
	if not tfmt:
		tfmt = 'Television/%TITLE%/%TITLE% - S%SEASON%E%EPISODEPAD% - %SUBTITLE%'
	mfmt = db.getSetting('mythvideo.MOVIEexportfmt')
	if not mfmt:
		mfmt = 'Movies/%TITLE%'
	gfmt = db.getSetting('mythvideo.GENERICexportfmt')
	if not gfmt:
		gfmt = 'Videos/%TITLE%'
	print "Current output formats:"
	print "    TV:      "+tfmt
	print "    Movies:  "+mfmt
	print "    Generic: "+gfmt

def main():
	parser = OptionParser(usage="usage: %prog [options] [jobid]")

	parser.add_option("-f", "--helpformat", action="store_true", default=False, dest="fmthelp",
			help="Print explination of file format string.")
	parser.add_option("-p", "--printformat", action="store_true", default=False, dest="fmtprint",
			help="Print current file format string.")
	parser.add_option("--tformat", action="store", type="string", dest="tformat",
			help="Use TV format for current task. If no task, store in database.")
	parser.add_option("--mformat", action="store", type="string", dest="mformat",
			help="Use Movie format for current task. If no task, store in database.")
	parser.add_option("--gformat", action="store", type="string", dest="gformat",
			help="Use Generic format for current task. If no task, store in database.")
	parser.add_option("--chanid", action="store", type="int", dest="chanid",
			help="Use chanid for manual operation")
	parser.add_option("--starttime", action="store", type="int", dest="starttime",
			help="Use starttime for manual operation")
	parser.add_option("--listingonly", action="store_true", default=False, dest="listingonly",
			help="Use data from listing provider, rather than grabber")
	parser.add_option("-s", "--simulation", action="store_true", default=False, dest="sim",
			help="Simulation (dry run), no files are copied or new entries made")
	parser.add_option("--skip", action="store_true", default=False, dest="skip") # debugging use only

	opts, args = parser.parse_args()

	if opts.fmthelp:
		usage_format()
		sys.exit(0)

	if opts.fmtprint:
		usage_current()
		sys.exit(0)

	if opts.chanid and opts.starttime:
		export = VIDEO(opts,0)
	elif len(args) == 1:
		export = VIDEO(opts,int(args[0]))
	else:
		if opts.tformat or opts.mformat or opts.gformat:
			db = MythDB()
			if opts.tformat:
				print "Changing TV format to: "+opts.tformat
				db.setSetting('mythvideo.TVexportfmt',opts.tformat)
			if opts.mformat:
				print "Changing Movie format to: "+opts.mformat
				db.setSetting('mythvideo.MOVIEexportfmt',opts.mformat)
			if opts.gformat:
				print "Changing Generic format to: "+opts.gformat
				db.setSetting('mythvideo.GENERICexportfmt',opts.gformat)
			sys.exit(0)
		else:
			parser.print_help()
			sys.exit(2)

if __name__ == "__main__":
	main()
